function [D]=mol3D_psisolve(h_col,b_col,sig_v,delta_v,lambda_v,...
            dx,dy,D1,D2,D3,D4,D5,psi_i_y,m,n,num_it,tol_v,J_E,F_E)
                                                            
% Create the diffusion matrix by using Kronekar delta products.
 % Create Xplus matrix.
XPLUS = D5 + D1;
% Create Xminus matrix.
XMINUS = D5 + D2;
% Same for y.
YPLUS = D5 + D4; YMINUS = D5 + D3;
% Create Backward Difference in x.
XBDIFF = D5 - D2;
% Create h3 column.
h3col = h_col.^3;
% Sup diagonal.
A1 = (0.5).*(XPLUS)*(h3col);
% Sub diagonal .
A2 =  (0.5).*(XMINUS)*(h3col);
% Sup sup diagonal.
A4 = (0.5)*((dx^2)/(dy^2)).*(YPLUS)*(h3col);
% Sub sub diagaonl.
A3 = (0.5)*((dx^2)/(dy^2)).*(YMINUS)*(h3col);
% Diagonal term.
A5 = -(A1 + A2 + A3 + A4);

% Matrix multiply to put a5 onto the diagonal.
D5a = spdiags(A5,0,n*m,n*m)*D5;

% Super diagonal terms.
D1a = (spdiags(A1,0,m*n,m*n))*D1;

% Sub diagonal terms.
D2a = (spdiags(A2,0,m*n,m*n))*D2;

% Super super diagonal terms.
D4a = spdiags(A4,0,m*n,m*n)*D4;

% Sub sub diagonal terms.
D3a = spdiags(A3,0,m*n,m*n)*D3;

% Now add them all together for total matrix.
DT = D1a + D2a + D3a + D4a + D5a;

% Insert into total matrix.
J_E(psi_i_y,psi_i_y) = DT;

% Now create the source term for Psi.
S_t = ((dx)*sig_v/(1)).*(XBDIFF)*(h3col);

% Construct mean condition on RHS.
FPsi_mc = delta_v.*((sum(h_col))) + (sum(b_col)) - ...
                (1./lambda_v).*(sum(h_col.^(-1)));
% Add into source.
F_E(psi_i_y) = S_t;

F = F_E;
J = J_E;

[D,flag,relres,iter,resvec] = minres(J,F,tol_v,num_it); 

if flag==1
    fprintf('Did not reach tolerance in minres\n')
    if relres>1e-6
        error('Tolerance larger than 1e-6')
    end
end

% Determine constant to satisfy the mean condition.
Cmean = (1./(m*n))*(FPsi_mc - sum(D));
D = D + Cmean;

end


